#include <stdio.h>
#include <jsapi.h>


static JSBool my_save_to_file(JSContext *ctx, JSObject *obj,
                              uintN argc, jsval *argv, jsval *rval)
{
    const char *fname, *text;
    FILE *fp;
    
    fname = JS_GetStringBytes(JS_ValueToString(ctx, argv[0]));
    text = JS_GetStringBytes(JS_ValueToString(ctx, argv[1]));

    fp = fopen(fname, "w");
    fprintf(fp, text);
    fclose(fp);

    return JS_TRUE;
}

static JSFunctionSpec my_functions[] = {
    { "save_to_file", my_save_to_file, 2, 0, 0 },
    { 0, 0, 0, 0, 0 },
};



static JSBool global_enumerate(JSContext *cx, JSObject *obj)
{
    return JS_EnumerateStandardClasses(cx, obj);
}


static JSBool global_resolve(JSContext *cx, JSObject *obj, jsval id, uintN flags,
                             JSObject **objp)
{
    return JS_TRUE;
}

JSClass global_class = {
   "global", JSCLASS_NEW_RESOLVE,
    JS_PropertyStub,  JS_PropertyStub,
    JS_PropertyStub,  JS_PropertyStub,
    global_enumerate, (JSResolveOp)global_resolve,
    JS_ConvertStub,   JS_FinalizeStub
};


int initialize_js_runtime(JSRuntime **rt, JSContext **ctx, JSObject **global)
{
    JSVersion version;
    JSBool result;

    *rt = JS_NewRuntime(64L * 1024L * 1024L);
    if (!*rt) {
        fprintf(stderr, "cannot create js runtime\n");
        return 0;
    }
    *ctx = JS_NewContext(*rt, 8192);
    if (!*ctx) {
        fprintf(stderr, "cannot create js context\n");
        return 0;
    }
    *global = JS_NewObject(*ctx, &global_class, NULL, NULL);
    if (!*global) {
        fprintf(stderr, "cannot create global objects\n");
        return 0;
    }

    result = JS_InitStandardClasses(*ctx, *global);
    if (!result) {
        fprintf(stderr, "cannot init standard classes\n");
        return 0;
    }

    JS_DefineFunctions(*ctx, *global, my_functions);

    return 1;
}

int main(int argc, char *argv[])
{
    JSRuntime *rt = NULL;
    JSContext *ctx = NULL;
    JSObject *global = NULL;
    const char *script;
    const char filename[] = "(inline)";
    uintN lineno = 0; 
    jsval rval;
    JSString *str = NULL;
    JSBool ok;

    if (argc < 2) {
        printf("Usage: inlinejs 'script'\n");
        exit(-1);
    }
    script = argv[1];

    if (!initialize_js_runtime(&rt, &ctx, &global)) {
        fprintf(stderr, "cannot initialized JavaScript env\n");
        exit(-1);
    }

    if (!JS_EvaluateScript(ctx, global, script, strlen(script),
                           filename, lineno, &rval))
    {
        fprintf(stderr, "cannot exec JavaScript\n");
        exit(1);
    }
    str = JS_ValueToString(ctx, rval);

    JS_DestroyContext(ctx);
    JS_DestroyRuntime(rt); 
    return 0;
}
